@@ -37,7 +37,7 @@
 extern "C" {
 #endif
 
-#define LIBGNUTLS_VERSION "0.9.0"
+#define LIBGNUTLS_VERSION "0.9.1"
 
 /* For backwards compatibility */
 #include <gnutls/compat4.h>
@@ -63,11 +63,11 @@ typedef enum gnutls_kx_algorithm { GNUTLS_KX_RSA=1, GNUTLS_KX_DHE_DSS,
 
 typedef enum gnutls_credentials_type { GNUTLS_CRD_CERTIFICATE=1, GNUTLS_CRD_ANON, GNUTLS_CRD_SRP } gnutls_credentials_type;
 
-typedef enum gnutls_mac_algorithm { GNUTLS_MAC_NULL=1, GNUTLS_MAC_MD5, GNUTLS_MAC_SHA } gnutls_mac_algorithm;
+typedef enum gnutls_mac_algorithm { GNUTLS_MAC_NULL=1, GNUTLS_MAC_MD5, GNUTLS_MAC_SHA, GNUTLS_MAC_MD2 } gnutls_mac_algorithm;
 
 /* The enumerations here should have the same value with gnutls_mac_algorithm.
  */
-typedef enum gnutls_digest_algorithm { GNUTLS_DIG_NULL=1, GNUTLS_DIG_MD5, GNUTLS_DIG_SHA } gnutls_digest_algorithm;
+typedef enum gnutls_digest_algorithm { GNUTLS_DIG_NULL=1, GNUTLS_DIG_MD5, GNUTLS_DIG_SHA, GNUTLS_DIG_MD2 } gnutls_digest_algorithm;
 
 /* exported for other gnutls headers. This is the maximum number
  * of algorithms (ciphers, kx or macs). 
@@ -301,8 +301,11 @@ int gnutls_anon_allocate_client_credentials( gnutls_anon_server_credentials *sc)
 void gnutls_certificate_free_credentials( gnutls_certificate_credentials sc);
 int gnutls_certificate_allocate_credentials( gnutls_certificate_credentials *sc);
 
-int gnutls_certificate_set_dh_params(gnutls_certificate_credentials res, gnutls_dh_params);
-int gnutls_certificate_set_rsa_params(gnutls_certificate_credentials res, gnutls_rsa_params rsa_params);
+void gnutls_certificate_free_keys(gnutls_certificate_credentials sc);
+
+void gnutls_certificate_set_dh_params(gnutls_certificate_credentials res, gnutls_dh_params);
+void gnutls_certificate_set_rsa_params(gnutls_certificate_credentials res, gnutls_rsa_params rsa_params);
+void gnutls_certificate_set_verify_flags(gnutls_certificate_credentials res, unsigned int flags);
 
 int gnutls_certificate_set_x509_trust_file( gnutls_certificate_credentials res, const char* CAFILE, 
 	gnutls_x509_crt_fmt);
@@ -399,7 +402,7 @@ void gnutls_openpgp_send_key(gnutls_session session, gnutls_openpgp_key_status s
 
 /* fingerprint */
 int gnutls_fingerprint(gnutls_digest_algorithm algo, const gnutls_datum* data, 
-	char* result, size_t* result_size);
+	unsigned char* result, int* result_size);
 
 #ifndef GNUTLS_UI_H
 # define GNUTLS_UI_H
