@@ -74,7 +74,7 @@ static int gen_rsa_export_server_kx(gnutls_session session, opaque ** data)
 	uint8 *data_e, *data_m;
 	int ret = 0, data_size;
 	gnutls_cert *apr_cert_list;
-	gnutls_private_key *apr_pkey;
+	gnutls_privkey* apr_pkey;
 	int apr_cert_list_length;
 	gnutls_datum signature, ddata;
 	CERTIFICATE_AUTH_INFO info;
