@@ -1,5 +1,5 @@
 /*
- *      Copyright (C) 2000,2002 Nikos Mavroyanopoulos
+ *      Copyright (C) 2000,2002,2003 Nikos Mavroyanopoulos
  *
  * This file is part of GNUTLS.
  *
@@ -33,41 +33,22 @@ void _gnutls_dump_mpi(char* prefix, GNUTLS_MPI a)
 	
 	if (gcry_mpi_print(GCRYMPI_FMT_HEX, buf, &n, a))
 		strcpy(buf, "[can't print value]"); /* Flawfinder: ignore */
-	_gnutls_log( "GNUTLS_MPI: length: %d\n\t%s%s\n", (n-1)/2, prefix, buf);
-}
-
-
-char *_gnutls_bin2hex(const unsigned char *old, const size_t oldlen)
-{
-	unsigned char *new = NULL;
-	int i, j;
-
-	new = calloc(1, oldlen * 2 * sizeof(char) + 1);
-	if (!new)
-		return (new);
-
-	for (i = j = 0; i < oldlen; j+=2) {
-		sprintf(&new[j], "%.2x", old[i]); /* Flawfinder: ignore */
-		i++;
-	}
-	new[j] = '\0';
-
-	return (new);
+	_gnutls_debug_log( "GNUTLS_MPI: length: %d\n\t%s%s\n", (n-1)/2, prefix, buf);
 }
 
 void _gnutls_print_state(gnutls_session session)
 {
 
-	_gnutls_log( "GNUTLS State:\n");
-	_gnutls_log( "Connection End: %d\n",
+	_gnutls_debug_log( "GNUTLS State:\n");
+	_gnutls_debug_log( "Connection End: %d\n",
 		session->security_parameters.entity);
-	_gnutls_log( "Cipher Algorithm: %d\n",
+	_gnutls_debug_log( "Cipher Algorithm: %d\n",
 		session->security_parameters.read_bulk_cipher_algorithm);
-	_gnutls_log( "MAC algorithm: %d\n",
+	_gnutls_debug_log( "MAC algorithm: %d\n",
 		session->security_parameters.read_mac_algorithm);
-	_gnutls_log( "Compression Algorithm: %d\n",
+	_gnutls_debug_log( "Compression Algorithm: %d\n",
 		session->security_parameters.read_compression_algorithm);
-	_gnutls_log( "\n");
+	_gnutls_debug_log( "\n");
 
 }
 
@@ -140,17 +121,3 @@ static char str[512];
 }
 #endif
 
-#ifdef DEBUG
-int gnutls_x509_extract_certificate_dn( const gnutls_datum*, gnutls_x509_dn*);
-
-const char* GET_CN( gnutls_datum cert) {
-static gnutls_x509_dn dn;
-
-	if (gnutls_x509_extract_certificate_dn( &cert, &dn) >= 0)
-		return &dn.common_name[0];
-	else 
-		return NULL;
-
-}
-
-#endif
