@@ -162,6 +162,7 @@ typedef struct gnutls_hash_entry gnutls_hash_entry;
 static const gnutls_hash_entry hash_algorithms[] = {
 	{"SHA", GNUTLS_MAC_SHA, 20},
 	{"MD5", GNUTLS_MAC_MD5, 16},
+	{"MD2", GNUTLS_MAC_MD2, 16},
 	{"NULL", GNUTLS_MAC_NULL, 0},
 	{0, 0, 0}
 };
@@ -1056,7 +1057,7 @@ _gnutls_qsort(gnutls_session session, void *_base, size_t nmemb, size_t size,
 #ifdef DEBUG
 	if (size > MAX_ELEM_SIZE) {
 		gnutls_assert();
-		_gnutls_log( "QSORT BUG\n");
+		_gnutls_debug_log( "QSORT BUG\n");
 		exit(1);
 	}
 #endif
@@ -1152,9 +1153,9 @@ _gnutls_supported_ciphersuites_sorted(gnutls_session session,
 	}
 
 #ifdef SORT_DEBUG
-	_gnutls_log( "Unsorted: \n");
+	_gnutls_debug_log( "Unsorted: \n");
 	for (i = 0; i < count; i++)
-		_gnutls_log( "\t%d: %s\n", i,
+		_gnutls_debug_log( "\t%d: %s\n", i,
 			_gnutls_cipher_suite_get_name((*ciphers)[i]));
 #endif
 
@@ -1162,9 +1163,9 @@ _gnutls_supported_ciphersuites_sorted(gnutls_session session,
 		      sizeof(GNUTLS_CipherSuite), _gnutls_compare_algo);
 
 #ifdef SORT_DEBUG
-	_gnutls_log( "Sorted: \n");
+	_gnutls_debug_log( "Sorted: \n");
 	for (i = 0; i < count; i++)
-		_gnutls_log( "\t%d: %s\n", i,
+		_gnutls_debug_log( "\t%d: %s\n", i,
 			_gnutls_cipher_suite_get_name((*ciphers)[i]));
 #endif
 
