@@ -305,11 +305,15 @@ int _gnutls_auth_info_set( gnutls_session session, gnutls_credentials_type type,
  * opaque data.
  */
 int _gnutls_generate_key(GNUTLS_KEY key) {
-        _gnutls_mpi_print( NULL, &key->key.size, key->KEY);
-	key->key.data = gnutls_secure_malloc( key->key.size);
+size_t tmp;
+
+        _gnutls_mpi_print( NULL, &tmp, key->KEY);
+	key->key.data = gnutls_secure_malloc( tmp);
 	if ( key->key.data==NULL) {
 		return GNUTLS_E_MEMORY_ERROR;
 	}
-	_gnutls_mpi_print( key->key.data, &key->key.size, key->KEY);
+	_gnutls_mpi_print( key->key.data, &tmp, key->KEY);
+	
+	key->key.size = tmp;
 	return 0;
 }
