@@ -18,11 +18,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
-MPI gnutls_get_dh_params(gnutls_dh_params, MPI *ret_p, int bits);
+int _gnutls_get_dh_params(gnutls_dh_params, MPI *ret_p, MPI* ret_g);
 MPI gnutls_calc_dh_secret( MPI *ret_x, MPI g, MPI prime );
 MPI gnutls_calc_dh_key( MPI f, MPI x, MPI prime );
 int _gnutls_dh_generate_prime(MPI *ret_g, MPI* ret_n, int bits);
-void _gnutls_dh_clear_mpis(void);
-int _gnutls_dh_calc_mpis(void);
-
-extern _gnutls_dh_params _gnutls_dh_default_params;
