@@ -155,7 +155,7 @@ int gnutls_dh_params_import_raw(gnutls_dh_params dh_params, const gnutls_datum *
 int gnutls_dh_params_init(gnutls_dh_params * dh_params)
 {
 
-	(*dh_params) = gnutls_calloc(1, sizeof(gnutls_dh_params));
+	(*dh_params) = gnutls_calloc(1, sizeof(_gnutls_dh_params));
 	if (*dh_params == NULL) {
 		gnutls_assert();
 		return GNUTLS_E_MEMORY_ERROR;
@@ -227,7 +227,7 @@ int gnutls_dh_params_generate2(gnutls_dh_params params, int bits)
   * in prime and generator structures.
   *
   * If the structure is PEM encoded, it should have a header
-  * of "BEGIN DH PARAMETERS", and must be null terminated.
+  * of "BEGIN DH PARAMETERS".
   *
   * In case of failure a negative value will be returned, and
   * 0 on success.
@@ -397,7 +397,6 @@ int gnutls_dh_params_export_pkcs3( gnutls_dh_params params,
 	}
 
 	if (format == GNUTLS_X509_FMT_DER) {
-#warning FIX THIS
 		if ((result=asn1_der_coding( c2, "", params_data, params_data_size, NULL)) != ASN1_SUCCESS) {
 			gnutls_assert();
 			asn1_delete_structure(&c2);
