@@ -84,8 +84,11 @@ int _gnutls_mpi_print_lz( opaque *buffer, size_t *nbytes, const GNUTLS_MPI a ) {
  * from asn1 structs. Combines the read and mpi_scan
  * steps.
  */
-int _gnutls_x509_read_int( ASN1_TYPE node, const char* value, char* tmpstr, int tmpstr_size, GNUTLS_MPI* ret_mpi) {
+int _gnutls_x509_read_int( ASN1_TYPE node, const char* value, 
+	char* tmpstr, int tmpstr_size, GNUTLS_MPI* ret_mpi)
+{
 int len, result;
+size_t s_len;
 
 	len = tmpstr_size - 1;
 	result = asn1_read_value( node, value, tmpstr, &len);
@@ -94,7 +97,8 @@ int len, result;
 		return _gnutls_asn2err(result);
 	}
 
-	if (_gnutls_mpi_scan( ret_mpi, tmpstr, &len) != 0) {
+	s_len = len;
+	if (_gnutls_mpi_scan( ret_mpi, tmpstr, &s_len) != 0) {
 		gnutls_assert();
 		return GNUTLS_E_MPI_SCAN_FAILED;
 	}
@@ -102,13 +106,3 @@ int len, result;
 	return 0;
 }
 
-/* front end for asn1_create_element.
- */
-asn1_retCode _gnutls_asn1_create_element(ASN1_TYPE definitions,const char *source_name,
-                                 ASN1_TYPE *element, const char *dest_name) {
-
-	*element = ASN1_TYPE_EMPTY;
-	
-	return asn1_create_element( definitions, source_name, element, dest_name);
-                                 
-}
