@@ -212,7 +212,9 @@ const gnutls_datum *gnutls_certificate_get_ours(gnutls_session session)
 	   return NULL; /* no certificate */
 	}
 	
-	return &cred->cert_list[index][0].raw;
+	if (cred->ncerts > (unsigned int) index)
+		return &cred->cert_list[index][0].raw;
+	return NULL;
 }
 
 /**
@@ -285,12 +287,13 @@ int gnutls_certificate_client_get_request_status(gnutls_session session)
   * Returns a negative value in case of an error.
   *
   **/
-int gnutls_fingerprint(gnutls_digest_algorithm algo, const gnutls_datum* data, char* result, size_t* result_size)
+int gnutls_fingerprint(gnutls_digest_algorithm algo, const gnutls_datum* data, 
+	unsigned char* result, int* result_size)
 {
 	GNUTLS_HASH_HANDLE td;
 	int hash_len = _gnutls_hash_get_algo_len(algo);
 	
-	if (hash_len < 0 || (size_t)hash_len > *result_size ||
+	if (hash_len < 0 || hash_len > *result_size ||
 		result==NULL) 
 	{
 		*result_size = hash_len;
