@@ -39,7 +39,6 @@
 #include <gnutls_str.h>
 #include <debug.h>
 #include <x509_b64.h>
-#include <gnutls_privkey.h>
 #include <gnutls_x509.h>
 #include "x509/common.h"
 #include "x509/x509.h"
@@ -153,9 +152,10 @@ int _gnutls_x509_cert_verify_peers(gnutls_session session)
  */
 static int _gnutls_check_key_cert_match( gnutls_certificate_credentials res) 
 {
-int pk = res->cert_list[res->ncerts-1][0].subject_pk_algorithm;
+uint pk = res->cert_list[res->ncerts-1][0].subject_pk_algorithm;
 	
-	if (res->pkey[res->ncerts-1].pk_algorithm != (unsigned int)pk) {
+	if (res->pkey[res->ncerts-1].pk_algorithm != pk) 
+	{
 		gnutls_assert();
 		return GNUTLS_E_CERTIFICATE_KEY_MISMATCH;
 	}
@@ -307,40 +307,39 @@ static int parse_pkcs7_cert_mem( gnutls_cert** cert_list, int* ncerts, const
 static int parse_pem_cert_mem( gnutls_cert** cert_list, int* ncerts, 
 	const char *input_cert, int input_cert_size)
 {
-	int siz, siz2, i;
+	int size, siz2, i;
 	const char *ptr;
 	opaque *ptr2;
 	gnutls_datum tmp;
 	int ret, count;
 
-	if ( (ptr = strstr( input_cert, PEM_PKCS7_SEP)) != NULL) 
+	if ( (ptr = strnstr( input_cert, PEM_PKCS7_SEP, input_cert_size)) != NULL) 
 	{
-		siz = strlen( ptr);
+		size = strlen( ptr);
 
 		ret = parse_pkcs7_cert_mem( cert_list, ncerts, ptr,
-			siz, CERT_PEM);
+			size, CERT_PEM);
 
 		return ret;
 	}
 
 	/* move to the certificate
 	 */
-	ptr = strstr( input_cert, PEM_CERT_SEP);
-	if (ptr == NULL) ptr = strstr( input_cert, PEM_CERT_SEP2);
+	ptr = strnstr( input_cert, PEM_CERT_SEP, input_cert_size);
+	if (ptr == NULL) ptr = strnstr( input_cert, PEM_CERT_SEP2, input_cert_size);
 
 	if (ptr == NULL) {
 		gnutls_assert();
 		return GNUTLS_E_BASE64_DECODING_ERROR;
 	}
-	siz = strlen( ptr);
+	size = input_cert_size - (ptr - input_cert);
 
 	i = *ncerts + 1;
 	count = 0;
 
 	do {
 
-		siz2 = _gnutls_fbase64_decode(NULL, ptr, siz, &ptr2);
-		siz -= siz2;
+		siz2 = _gnutls_fbase64_decode(NULL, ptr, size, &ptr2);
 
 		if (siz2 < 0) {
 			gnutls_assert();
@@ -372,8 +371,16 @@ static int parse_pem_cert_mem( gnutls_cert** cert_list, int* ncerts,
 		ptr++;
 		/* find the next certificate (if any)
 		 */
-		ptr = strstr(ptr, PEM_CERT_SEP);
-		if (ptr == NULL) ptr = strstr( input_cert, PEM_CERT_SEP2);
+		size = input_cert_size - (ptr - input_cert);
+		
+		if (size > 0) {
+			char* ptr2;
+			
+			ptr2 = strnstr(ptr, PEM_CERT_SEP, size);
+			if (ptr2 == NULL) ptr2 = strnstr( ptr, PEM_CERT_SEP2, size);
+			
+			ptr = ptr2;
+		} else ptr = NULL;
 
 		i++;
 		count++;
@@ -430,76 +437,42 @@ int read_cert_mem(gnutls_certificate_credentials res, const char *cert, int cert
 }
 
 
-
-/* This will check if the given DER key is a PKCS-1 RSA key.
- * Returns 0 if the key is an RSA one.
- */
-int _gnutls_der_check_if_rsa_key(const gnutls_datum * key_struct)
+static int privkey_cpy( gnutls_privkey* dest, gnutls_x509_privkey src)
 {
-	ASN1_TYPE c2;
-	int result;
+int i, ret;
 
-	if (key_struct->size == 0 || key_struct->data == NULL) {
-		gnutls_assert();
-		return GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE;
+	memset( dest, 0, sizeof(gnutls_privkey));
+
+	for(i=0;i<src->params_size;i++) {
+		dest->params[i] = _gnutls_mpi_copy( src->params[i]);
+		if (dest->params[i] == NULL) {
+			gnutls_assert();
+			ret = GNUTLS_E_MEMORY_ERROR;
+			goto cleanup;
+		}
 	}
 
-	if ((result=asn1_create_element
-	    (_gnutls_get_gnutls_asn(), "GNUTLS.RSAPrivateKey", &c2
-	       )) != ASN1_SUCCESS) 
-	{
-		gnutls_assert();
-		return _gnutls_asn2err(result);
-	}
-
-	result = asn1_der_decoding(&c2, key_struct->data, key_struct->size, NULL);
-	asn1_delete_structure(&c2);
-
-	if (result != ASN1_SUCCESS) {
-		/* couldn't decode DER */
-	
-		gnutls_assert();
-		return _gnutls_asn2err(result);
-	}
+	dest->pk_algorithm = src->pk_algorithm;
+	dest->params_size = src->params_size;
 
 	return 0;
-}
 
-/* This will check if the given DER key is an openssl formated DSA key.
- * Returns 0 if the key is a DSA one.
- */
-int _gnutls_der_check_if_dsa_key(const gnutls_datum * key_struct)
-{
-	ASN1_TYPE c2;
-	int result;
-
-	if (key_struct->size == 0 || key_struct->data == NULL) {
-		gnutls_assert();
-		return GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE;
-	}
-
-	if ((result=asn1_create_element
-	    (_gnutls_get_gnutls_asn(), "GNUTLS.DSAPrivateKey", &c2
-	       )) != ASN1_SUCCESS) 
-	{
-		gnutls_assert();
-		return _gnutls_asn2err(result);
-	}
-
-	result = asn1_der_decoding(&c2, key_struct->data, key_struct->size, NULL);
-	asn1_delete_structure(&c2);
-
-	if (result != ASN1_SUCCESS) {
-		/* couldn't decode DER */
+	cleanup:
 	
-		gnutls_assert();
-		return _gnutls_asn2err(result);
+	for (i=0;i<src->params_size;i++) {
+		_gnutls_mpi_release( &dest->params[i]);
 	}
-
-	return 0;
+	return ret;
 }
 
+void _gnutls_privkey_deinit(gnutls_privkey *key)
+{
+int i;
 
+	for (i = 0; i < key->params_size; i++) {
+		_gnutls_mpi_release( &key->params[i]);
+	}
+}
 
 
 /* Reads a PEM encoded PKCS-1 RSA private key from memory
@@ -510,102 +483,38 @@ static int read_key_mem(gnutls_certificate_credentials res, const char *key, int
 	gnutls_x509_crt_fmt type)
 {
 	int ret;
-	opaque *b64 = NULL;
 	gnutls_datum tmp;
-	gnutls_pk_algorithm pk;
+	gnutls_x509_privkey tmpkey;
 
 	/* allocate space for the pkey list
 	 */
-	res->pkey = gnutls_realloc_fast( res->pkey, (res->ncerts+1)*sizeof(gnutls_private_key));
+	res->pkey = gnutls_realloc_fast( res->pkey, (res->ncerts+1)*sizeof(gnutls_privkey));
 	if (res->pkey==NULL) {
 		gnutls_assert();
 		return GNUTLS_E_MEMORY_ERROR;
 	}
 
-	/* read PKCS-1 private key */
-
-	if (type==GNUTLS_X509_FMT_DER) { /* DER */
-		int cv;
-		
-		tmp.data = (opaque*)key;
-		tmp.size = key_size;
-
-		/* The only way to distinguish the keys
-		 * is to count the sequence of integers.
-		 */
-		pk = GNUTLS_PK_UNKNOWN;
-		cv = _gnutls_der_check_if_rsa_key( &tmp);
-		if (cv==0)
-			pk = GNUTLS_PK_RSA;
-		else {
-		   cv = _gnutls_der_check_if_dsa_key( &tmp);
-		   if (cv == 0) pk = GNUTLS_PK_DSA;
-	        }
-
-	} else { /* PEM */
-
-		/* If we find the "DSA PRIVATE" string in the
-		 * pem encoded certificate then it's a DSA key.
-		 */
-		if (strstr( key, "DSA PRIVATE")!=NULL) {
-			pk = GNUTLS_PK_DSA;
-			key = strstr( key, PEM_KEY_DSA_SEP);
-			if (key == NULL) {
-				gnutls_assert();
-				return GNUTLS_E_BASE64_DECODING_ERROR;
-			}			key_size = strlen( key);
-		} else {
-			pk = GNUTLS_PK_RSA;
-			key = strstr( key, PEM_KEY_RSA_SEP);
-			if (key == NULL) {
-				gnutls_assert();
-				return GNUTLS_E_BASE64_DECODING_ERROR;
-			}
-			key_size = strlen( key);
-		}
-			
-
-		ret = _gnutls_fbase64_decode( NULL, key, key_size, &b64);
-
-		if (ret < 0) {
-			gnutls_assert();
-			return GNUTLS_E_BASE64_DECODING_ERROR;
-		}
-
-		tmp.data = b64;
-		tmp.size = ret;
+	ret = gnutls_x509_privkey_init( &tmpkey); //res->pkey[res->ncerts]);
+	if (ret < 0) {
+		gnutls_assert();
+		return ret;
 	}
 
-	switch (pk) { /* decode the key */
-		case GNUTLS_PK_RSA:
-			if ((ret =
-			     _gnutls_PKCS1key2gnutlsKey(&res->pkey[res->ncerts],
-						tmp)) < 0) {
-				gnutls_assert();
-				gnutls_free(b64);
-				return ret;
-			}
-			break;
-		case GNUTLS_PK_DSA:
-			if ((ret =
-			     _gnutls_DSAkey2gnutlsKey(&res->pkey[res->ncerts],
-							tmp)) < 0) {
-				gnutls_assert();
-				gnutls_free(b64);
-				return ret;
-			}
-			break;
-		default:
-			gnutls_assert();
-			gnutls_free(b64);
-			return GNUTLS_E_INTERNAL_ERROR;
+	tmp.data = (opaque*)key;
+	tmp.size = key_size;
+
+	ret = gnutls_x509_privkey_import( tmpkey, &tmp, type);
+	if (ret < 0) {
+		gnutls_assert();
+		gnutls_x509_privkey_deinit( tmpkey);
+
+		return ret;
 	}
 
-	/* this doesn't hurt in the DER case, since
-	 * b64 is NULL
-	 */
-	gnutls_free(b64);
-	
+	privkey_cpy( &res->pkey[res->ncerts], tmpkey);
+
+	gnutls_x509_privkey_deinit( tmpkey);
+
 	return 0;
 }
 
@@ -617,36 +526,68 @@ static int read_key_mem(gnutls_certificate_credentials res, const char *key, int
 #include <fcntl.h>
 #include <errno.h>
 
-opaque * _gnutls_file_to_str( const char * file, size_t* str_size)
+#ifdef HAVE_MMAP
+# include <unistd.h>
+# include <sys/mman.h>
+#endif
+
+typedef struct {
+	opaque * data;
+	size_t size;
+	int mmaped;
+} strfile;
+
+inline static void _strfile_free( strfile *x)
+{
+	if (x->mmaped) return;
+	
+	gnutls_free( x->data);
+	x->data = NULL;
+}
+
+strfile _gnutls_file_to_str( const char * file)
 {
 	int fd1 = -1;
-	opaque * ret = NULL;
 	struct stat stat_st;
 	size_t tot_size;
 	size_t left;
+	opaque* tmp;
 	ssize_t i = 0;
+	strfile null = { NULL, 0, 0 };
+	strfile ret = { NULL, 0, 0 };
 	
 	fd1 = open( file, 0);
 	if (fd1==-1) {
 		gnutls_assert();
-		return NULL;
+		return null;
 	}
 	
 	if (fstat( fd1, &stat_st) == -1) {
 		gnutls_assert();
 		goto error;
 	}
-	
+
 	tot_size = stat_st.st_size;
-	ret = gnutls_malloc( tot_size + 1);
-	if (ret == NULL) {
+
+#ifdef HAVE_MMAP
+	if ((tmp=mmap( NULL, tot_size, PROT_READ, MAP_SHARED, fd1, 0)) != MAP_FAILED) {
+		ret.mmaped = 1;
+		ret.data = tmp;
+		ret.size = tot_size;
+		
+		return ret;
+	}
+#endif
+
+	ret.data = gnutls_malloc( tot_size);
+	if (ret.data == NULL) {
 		gnutls_assert();
 		goto error;
 	}
 	
 	left = tot_size;
 	while (left > 0) {
-		i = read( fd1, &ret[tot_size - left], left);
+		i = read( fd1, &ret.data[tot_size - left], left);
 		if (i == -1) {
 			if (errno == EAGAIN || errno == EINTR)
 				continue;
@@ -659,18 +600,20 @@ opaque * _gnutls_file_to_str( const char * file, size_t* str_size)
 		left -= i;
 	}
 
-	close(fd1);
+	ret.size = tot_size - left;
 
-	*str_size = tot_size - left;
-	ret[*str_size] = 0; /* null terminated */
+	ret.mmaped = 0;
+
+	close(fd1);
 
 	return ret;
 	
 	error:
-		gnutls_free( ret);
+			
+		if (!ret.mmaped)
+			gnutls_free( ret.data);
 		close(fd1);
-		return NULL;
-		
+		return null;
 }
 
 /* Reads a certificate file
@@ -678,18 +621,17 @@ opaque * _gnutls_file_to_str( const char * file, size_t* str_size)
 static int read_cert_file(gnutls_certificate_credentials res, const char *certfile,
 	gnutls_x509_crt_fmt type)
 {
-	size_t size;
 	int ret;
-	char *x;
+	strfile x;
 
-	x = _gnutls_file_to_str( certfile, &size);
-	if (x == NULL) {
+	x = _gnutls_file_to_str( certfile);
+	if (x.data == NULL) {
 		gnutls_assert();
 		return GNUTLS_E_FILE_ERROR;
 	}
 
-	ret = read_cert_mem( res, x, size, type);
-	gnutls_free(x);
+	ret = read_cert_mem( res, x.data, x.size, type);
+	_strfile_free(&x);
 	
 	return ret;
 
@@ -703,22 +645,58 @@ static int read_cert_file(gnutls_certificate_credentials res, const char *certfi
 static int read_key_file(gnutls_certificate_credentials res, const char *keyfile,
 	gnutls_x509_crt_fmt type)
 {
-	size_t size;
 	int ret;
-	opaque* x;
+	strfile x;
 
-	x = _gnutls_file_to_str( keyfile, &size);
-	if (x == NULL) {
+	x = _gnutls_file_to_str( keyfile);
+	if (x.data == NULL) {
 		gnutls_assert();
 		return GNUTLS_E_FILE_ERROR;
 	}
 
-	ret = read_key_mem( res, x, size, type);
-	gnutls_free(x);
+	ret = read_key_mem( res, x.data, x.size, type);
+	_strfile_free(&x);
 	
 	return ret;
 }
 
+/**
+  * gnutls_certificate_free_keys - Used to free all the keys from a gnutls_certificate_credentials structure
+  * @sc: is an &gnutls_certificate_credentials structure.
+  *
+  * This function will delete all the keys and the certificates associated
+  * with the given credentials. This function must not be called when a
+  * TLS negotiation that uses the credentials is in progress.
+  *
+  **/
+void gnutls_certificate_free_keys(gnutls_certificate_credentials sc)
+{
+	uint i, j;
+
+	for (i = 0; i < sc->ncerts; i++) {
+		for (j = 0; j < sc->cert_list_length[i]; j++) {
+			_gnutls_free_cert( &sc->cert_list[i][j]);
+		}
+		gnutls_free( sc->cert_list[i]);
+	}
+
+	gnutls_free(sc->cert_list_length);
+	sc->cert_list_length = NULL;
+
+	gnutls_free(sc->cert_list);
+	sc->cert_list = NULL;
+
+	for (i = 0; i < sc->ncerts; i++) {
+		_gnutls_privkey_deinit( &sc->pkey[i]);
+	}
+
+	gnutls_free( sc->pkey);
+	sc->pkey = NULL;
+	
+	sc->ncerts = 0;
+
+}
+
 /**
   * gnutls_certificate_set_x509_key_mem - Used to set keys in a gnutls_certificate_credentials structure
   * @res: is an &gnutls_certificate_credentials structure.
@@ -923,21 +901,21 @@ int _gnutls_check_key_usage( const gnutls_cert* cert,
 static int parse_pem_ca_mem( gnutls_x509_crt** cert_list, int* ncerts, 
 	const char *input_cert, int input_cert_size)
 {
-	int siz, i;
+	int i, size;
 	const char *ptr;
 	gnutls_datum tmp;
 	int ret, count;
 
 	/* move to the certificate
 	 */
-	ptr = strstr( input_cert, PEM_CERT_SEP);
-	if (ptr == NULL) ptr = strstr( input_cert, PEM_CERT_SEP2);
+	ptr = strnstr( input_cert, PEM_CERT_SEP, input_cert_size);
+	if (ptr == NULL) ptr = strnstr( input_cert, PEM_CERT_SEP2, input_cert_size);
 
 	if (ptr == NULL) {
 		gnutls_assert();
 		return GNUTLS_E_BASE64_DECODING_ERROR;
 	}
-	siz = strlen( ptr);
+	size = input_cert_size - (ptr - input_cert);
 
 	i = *ncerts + 1;
 	count = 0;
@@ -961,7 +939,7 @@ static int parse_pem_ca_mem( gnutls_x509_crt** cert_list, int* ncerts,
 		}
 		
 		tmp.data = (char*)ptr;
-		tmp.size = siz;
+		tmp.size = size;
 	
 		ret =
 		     gnutls_x509_crt_import(
@@ -977,8 +955,17 @@ static int parse_pem_ca_mem( gnutls_x509_crt** cert_list, int* ncerts,
 		ptr++;
 		/* find the next certificate (if any)
 		 */
-		ptr = strstr(ptr, PEM_CERT_SEP);
-		if (ptr == NULL) ptr = strstr( input_cert, PEM_CERT_SEP2);
+
+		size = input_cert_size - (ptr - input_cert);
+		
+		if (size > 0) {
+			char* ptr2;
+			
+			ptr2 = strnstr(ptr, PEM_CERT_SEP, size);
+			if (ptr2 == NULL) ptr = strnstr( ptr, PEM_CERT_SEP2, size);
+			
+			ptr = ptr2;
+		} else ptr = NULL;
 
 		i++;
 		count++;
@@ -1084,24 +1071,22 @@ int gnutls_certificate_set_x509_trust_file(gnutls_certificate_credentials res,
 		const char *cafile, gnutls_x509_crt_fmt type)
 {
 	int ret, ret2;
-	size_t size;
-	opaque *x;
+	strfile x;
 
-	x = _gnutls_file_to_str( cafile, &size);
-	if (x == NULL) {
+	x = _gnutls_file_to_str( cafile);
+	if (x.data == NULL) {
 		gnutls_assert();
 		return GNUTLS_E_FILE_ERROR;
 	}
-
 	
 	if (type==GNUTLS_X509_FMT_DER)
 		ret = parse_der_ca_mem( &res->x509_ca_list, &res->x509_ncas,
-			x, size);
+			x.data, x.size);
 	else
 		ret = parse_pem_ca_mem( &res->x509_ca_list, &res->x509_ncas,
-			x, size);
+			x.data, x.size);
 
-	gnutls_free(x);
+	_strfile_free(&x);
 
 	if (ret < 0) {
 		gnutls_assert();
@@ -1117,20 +1102,20 @@ int gnutls_certificate_set_x509_trust_file(gnutls_certificate_credentials res,
 static int parse_pem_crl_mem( gnutls_x509_crl** crl_list, int* ncrls, 
 	const char *input_crl, int input_crl_size)
 {
-	int siz, i;
+	int size, i;
 	const char *ptr;
 	gnutls_datum tmp;
 	int ret, count;
 
 	/* move to the certificate
 	 */
-	ptr = strstr( input_crl, PEM_CRL_SEP);
+	ptr = strnstr( input_crl, PEM_CRL_SEP, input_crl_size);
 	if (ptr == NULL) {
 		gnutls_assert();
 		return GNUTLS_E_BASE64_DECODING_ERROR;
 	}
 
-	siz = strlen( ptr);
+	size = input_crl_size - (ptr - input_crl);
 
 	i = *ncrls + 1;
 	count = 0;
@@ -1154,7 +1139,7 @@ static int parse_pem_crl_mem( gnutls_x509_crl** crl_list, int* ncrls,
 		}
 		
 		tmp.data = (char*)ptr;
-		tmp.size = siz;
+		tmp.size = size;
 	
 		ret =
 		     gnutls_x509_crl_import(
@@ -1170,7 +1155,12 @@ static int parse_pem_crl_mem( gnutls_x509_crl** crl_list, int* ncrls,
 		ptr++;
 		/* find the next certificate (if any)
 		 */
-		ptr = strstr(ptr, PEM_CRL_SEP);
+
+		size = input_crl_size - (ptr - input_crl);
+
+		if (size > 0)
+			ptr = strnstr(ptr, PEM_CRL_SEP, size);
+		else ptr = NULL;
 		i++;
 		count++;
 
@@ -1297,23 +1287,22 @@ int gnutls_certificate_set_x509_crl_file(gnutls_certificate_credentials res,
 		const char *crlfile, gnutls_x509_crt_fmt type)
 {
 	int ret;
-	size_t size;
-	opaque * x;
+	strfile x;
 
-	x = _gnutls_file_to_str( crlfile, &size);
-	if (x == NULL) {
+	x = _gnutls_file_to_str( crlfile);
+	if (x.data == NULL) {
 		gnutls_assert();
 		return GNUTLS_E_FILE_ERROR;
 	}
 
 	if (type==GNUTLS_X509_FMT_DER)
 		ret = parse_der_crl_mem( &res->x509_crl_list, &res->x509_ncrls,
-			x, size);
+			x.data, x.size);
 	else
 		ret = parse_pem_crl_mem( &res->x509_crl_list, &res->x509_ncrls,
-			x, size);
+			x.data, x.size);
 	
-	gnutls_free(x);
+	_strfile_free(&x);
 
 	if (ret < 0) {
 		gnutls_assert();
