@@ -84,8 +84,10 @@ _asn1_add_node(unsigned int type)
 /* Function : _asn1_find_mode                                     */
 /* Description: searches an element called NAME starting from     */
 /*              POINTER. The name is composed by differents       */
-/*              identifiers separated by dot.The first identifier */
-/*              must be the name of *POINTER.                     */
+/*              identifiers separated by dots.When *POINTER has a */
+/*              name, the first identifier must be the name of    */
+/*              *POINTER, otherwise it must be the name of one    */
+/*              child of *POINTER.                                */
 /* Parameters:                                                    */
 /*   pointer: NODE_ASN element pointer.                           */
 /*   name: null terminated string with the element's name to find.*/
@@ -95,31 +97,40 @@ node_asn *
 _asn1_find_node(node_asn *pointer,const char *name)
 {
   node_asn *p;
-  char *n_end,n[128];
+  char *n_end,n[MAX_NAME_SIZE+1];
   const char *n_start;
 
-  if((name==NULL) || (name[0]==0)) return NULL;
+  if(pointer == NULL) return NULL;
 
-  n_start=name;
-  n_end=strchr(n_start,'.');     /* search the first dot */
-  if(n_end){
-    memcpy(n,n_start,n_end-n_start);
-    n[n_end-n_start]=0;
-    n_start=n_end;
-    n_start++;
-  }
-  else{
-    _asn1_str_cpy(n,sizeof(n),n_start);
-    n_start=NULL;
-  }
+  if(name==NULL) return NULL;
 
   p=pointer;
-  while(p){
-    if((p->name) && (!strcmp(p->name,n))) break;
-    else p=p->right;
-  } /* while */
+  n_start=name;
 
-  if(p==NULL) return NULL;
+  if(p->name != NULL){ /* has *pointer a name ? */
+    n_end=strchr(n_start,'.');     /* search the first dot */
+    if(n_end){
+      memcpy(n,n_start,n_end-n_start);
+      n[n_end-n_start]=0;
+      n_start=n_end;
+      n_start++;
+    }
+    else{
+      _asn1_str_cpy(n,sizeof(n),n_start);
+      n_start=NULL;
+    }
+    
+    while(p){
+      if((p->name) && (!strcmp(p->name,n))) break;
+      else p=p->right;
+    } /* while */
+    
+    if(p==NULL) return NULL;
+  }
+  else{ /* *pointer doesn't have a name */
+    if(n_start[0]==0)
+      return p;
+  }
 
   while(n_start){   /* Has the end of NAME been reached? */
     n_end=strchr(n_start,'.');    /* search the next dot */
@@ -757,4 +768,79 @@ _asn1_set_default_tag(ASN1_TYPE node)
 
 
 
+static const char*
+parse_version_number( const char *s, int *number )
+{
+    int val = 0;
+
+    if( *s == '0' && isdigit(s[1]) )
+	return NULL; /* leading zeros are not allowed */
+    for ( ; isdigit(*s); s++ ) {
+	val *= 10;
+	val += *s - '0';
+    }
+    *number = val;
+    return val < 0? NULL : s;
+}
+
+/* The parse version functions were copied from libgcrypt.
+ */
+static const char *
+parse_version_string( const char *s, int *major, int *minor, int *micro )
+{
+    s = parse_version_number( s, major );
+    if( !s || *s != '.' )
+	return NULL;
+    s++;
+    s = parse_version_number( s, minor );
+    if( !s || *s != '.' )
+	return NULL;
+    s++;
+    s = parse_version_number( s, micro );
+    if( !s )
+	return NULL;
+    return s; /* patchlevel */
+}
+
+/**
+  * asn1_check_version - This function checks the library's version
+  * @req_version: the version to check
+  *
+  * Check that the the version of the library is at minimum the requested one
+  * and return the version string; return NULL if the condition is not
+  * satisfied.  If a NULL is passed to this function, no check is done,
+  * but the version string is simply returned.
+  *
+  **/
+const char *
+asn1_check_version( const char *req_version )
+{
+    const char *ver = LIBTASN1_VERSION;
+    int my_major, my_minor, my_micro;
+    int rq_major, rq_minor, rq_micro;
+    const char *my_plvl, *rq_plvl;
+
+    if ( !req_version )
+	return ver;
+
+    my_plvl = parse_version_string( ver, &my_major, &my_minor, &my_micro );
+    if ( !my_plvl )
+	return NULL;  /* very strange our own version is bogus */
+    rq_plvl = parse_version_string( req_version, &rq_major, &rq_minor,
+								&rq_micro );
+    if ( !rq_plvl )
+	return NULL;  /* req version string is invalid */
+
+    if ( my_major > rq_major
+	|| (my_major == rq_major && my_minor > rq_minor)
+	|| (my_major == rq_major && my_minor == rq_minor
+				 && my_micro > rq_micro)
+	|| (my_major == rq_major && my_minor == rq_minor
+				 && my_micro == rq_micro
+				 && strcmp( my_plvl, rq_plvl ) >= 0) ) {
+	return ver;
+    }
+    return NULL;
+}
+
 
