@@ -17,8 +17,48 @@ typedef struct gnutls_x509_crt_int {
 	gnutls_pk_algorithm signature_algorithm;
 } gnutls_x509_crt_int;
 
+/* Raw encoded parameter.
+ */
+#define MAX_PARAMETER_SIZE 2400
+
+#define MAX_PRIV_PARAMS_SIZE 6 /* ok for RSA and DSA */
+
+/* parameters should not be larger than this limit */
+#define DSA_PRIVATE_PARAMS 5
+#define DSA_PUBLIC_PARAMS 4
+#define RSA_PRIVATE_PARAMS 6
+#define RSA_PUBLIC_PARAMS 2
+
+#if MAX_PRIV_PARAMS_SIZE - RSA_PRIVATE_PARAMS < 0
+# error INCREASE MAX_PRIV_PARAMS
+#endif
+
+#if MAX_PRIV_PARAMS_SIZE - DSA_PRIVATE_PARAMS < 0
+# error INCREASE MAX_PRIV_PARAMS
+#endif
+
 typedef struct gnutls_x509_privkey_int {
-	gnutls_datum raw; /* we only keep raw data for the moment */
+	MPI params[MAX_PRIV_PARAMS_SIZE];/* the size of params depends on the public 
+				 * key algorithm 
+				 */
+				/*
+				 * RSA: [0] is modulus
+				 *      [1] is public exponent
+				 *	[2] is private exponent
+				 *	[3] is prime1 (p)
+				 *	[4] is prime2 (q)
+				 *	[5] is coefficient (u == inverse of p mod q)
+				 * DSA: [0] is p
+				 *      [1] is q
+				 *      [2] is g
+				 *      [3] is y (public key)
+				 *      [4] is x (private key)
+				 */
+	int params_size; /* holds the number of params */
+
+	gnutls_pk_algorithm pk_algorithm;
+
+	ASN1_TYPE key;
 } gnutls_x509_privkey_int;
 
 typedef struct gnutls_x509_crt_int *gnutls_x509_crt;
