@@ -75,12 +75,19 @@ int _gnutls_gen_srp_server_kx(gnutls_session state, opaque ** data)
 	uint8 *data_n, *data_s;
 	uint8 *data_g, *username;
 	SRP_PWD_ENTRY *pwd_entry;
-	int err;
 	SRP_SERVER_AUTH_INFO info;
 	ssize_t data_size;
-	size_t n_b;
+	size_t n_b, tmp_size;
 	uint8 *data_b;
-	
+
+	if (state->security_parameters.extensions.srp_username[0] == 0) {
+		/* The peer didn't send a valid SRP extension with the
+		 * SRP username.
+		 */
+		gnutls_assert();
+		return GNUTLS_E_EMPTY_SRP_USERNAME;
+	}
+
 	if ( (ret=_gnutls_auth_info_set( state, GNUTLS_CRD_SRP, sizeof( SRP_SERVER_AUTH_INFO_INT), 1)) < 0) {
 		gnutls_assert();
 		return ret;
@@ -88,7 +95,7 @@ int _gnutls_gen_srp_server_kx(gnutls_session state, opaque ** data)
 
 	info = _gnutls_get_auth_info( state);
 	username = info->username;
-	
+
 	_gnutls_str_cpy( username, MAX_SRP_USERNAME, state->security_parameters.extensions.srp_username);
 
 	ret = _gnutls_srp_pwd_read_entry( state, username, &pwd_entry);
@@ -104,12 +111,14 @@ int _gnutls_gen_srp_server_kx(gnutls_session state, opaque ** data)
 		return GNUTLS_E_MPI_SCAN_FAILED;
 	}
 
-	if (_gnutls_mpi_scan( &N, pwd_entry->n.data, &pwd_entry->n.size) < 0) {
+	tmp_size = pwd_entry->n.size;
+	if (_gnutls_mpi_scan( &N, pwd_entry->n.data, &tmp_size) < 0) {
 		gnutls_assert();
 		return GNUTLS_E_MPI_SCAN_FAILED;
 	}
 
-	if (_gnutls_mpi_scan( &V, pwd_entry->v.data, &pwd_entry->v.size) < 0) {
+	tmp_size = pwd_entry->v.size;
+	if (_gnutls_mpi_scan( &V, pwd_entry->v.data, &tmp_size) < 0) {
 		gnutls_assert();
 		return GNUTLS_E_MPI_SCAN_FAILED;
 	}
@@ -338,6 +347,28 @@ int _gnutls_proc_srp_client_kx(gnutls_session state, opaque * data, size_t _data
 	return 0;
 }
 
+/* Checks if b%n==0 which is a fatal srp error.
+ * Returns a proper error code in that case, and 0 when
+ * all are ok.
+ */
+static int check_b_mod_n( GNUTLS_MPI b, GNUTLS_MPI n)
+{
+int ret;
+GNUTLS_MPI r = _gnutls_mpi_alloc_like(b);
+
+	_gnutls_mpi_mod( r, b, n);
+	ret = _gnutls_mpi_cmp_ui(r, 0);
+	
+	_gnutls_mpi_release( &r);
+
+	if (ret == 0) {
+		gnutls_assert();
+		return GNUTLS_E_RECEIVED_ILLEGAL_PARAMETER;
+	}
+	
+	return 0;
+}
+
 /* receive the key exchange message ( n, g, s, B) 
  */
 int _gnutls_proc_srp_server_kx(gnutls_session state, opaque * data, size_t _data_size)
@@ -431,6 +462,12 @@ int _gnutls_proc_srp_server_kx(gnutls_session state, opaque * data, size_t _data
 		gnutls_assert();
 		return GNUTLS_E_MPI_SCAN_FAILED;
 	}
+	
+	if ( (ret = check_b_mod_n( B, N)) < 0) {
+		gnutls_assert();
+		return ret;
+	}
+
 
 	/* generate x = SHA(s | SHA(U | ":" | p))
 	 * (or the equivalent using bcrypt)
